@@ -37,14 +37,21 @@ const preparseDirectiveHints = [];
 const originHints = [];
 let hintHelperRegistered = false;
 
-
 /******************************************************************************/
 
-let filterOnHeaders = false;
+CodeMirror.defineOption('trustedSource', false, (cm, state) => {
+    if ( typeof state !== 'boolean' ) { return; }
+    self.dispatchEvent(new CustomEvent('trustedSource', {
+        detail: state,
+    }));
+});
 
-CodeMirror.defineOption('filterOnHeaders', false, (cm, state) => {
-    filterOnHeaders = state;
-    self.dispatchEvent(new Event('filterOnHeaders'));
+CodeMirror.defineOption('trustedScriptletTokens', undefined, (cm, tokens) => {
+    if ( tokens === undefined || tokens === null ) { return; }
+    if ( typeof tokens[Symbol.iterator] !== 'function' ) { return; }
+    self.dispatchEvent(new CustomEvent('trustedScriptletTokens', {
+        detail: new Set(tokens),
+    }));
 });
 
 /******************************************************************************/
@@ -214,11 +221,15 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
         return '+';
     };
 
-    self.addEventListener('filterOnHeaders', ( ) => {
-        astParser.options.filterOnHeaders = filterOnHeaders;
+    self.addEventListener('trustedSource', ev => {
+        astParser.options.trustedSource = ev.detail;
     });
 
-    return {
+    self.addEventListener('trustedScriptletTokens', ev => {
+        astParser.options.trustedScriptletTokens = ev.detail;
+    });
+
+   return {
         lineComment: '!',
         token: function(stream) {
             if ( stream.sol() ) {
@@ -267,38 +278,6 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
             style = style.trim();
             return style !== '' ? style : null;
         },
-        setHints: function(details) {
-            if ( Array.isArray(details.redirectResources) ) {
-                for ( const [ name, desc ] of details.redirectResources ) {
-                    const displayText = desc.aliasOf !== ''
-                        ? `${name} (${desc.aliasOf})`
-                        : '';
-                    if ( desc.canRedirect ) {
-                        redirectNames.set(name, displayText);
-                    }
-                    if ( desc.canInject && name.endsWith('.js') ) {
-                        scriptletNames.set(name.slice(0, -3), displayText);
-                    }
-                }
-            }
-            if ( Array.isArray(details.preparseDirectiveEnv)) {
-                preparseDirectiveEnv.length = 0;
-                preparseDirectiveEnv.push(...details.preparseDirectiveEnv);
-            }
-            if ( Array.isArray(details.preparseDirectiveHints)) {
-                preparseDirectiveHints.push(...details.preparseDirectiveHints);
-            }
-            if ( Array.isArray(details.originHints) ) {
-                originHints.length = 0;
-                for ( const hint of details.originHints ) {
-                    originHints.push(hint);
-                }
-            }
-            if ( hintHelperRegistered === false ) {
-                hintHelperRegistered = true;
-                initHints();
-            }
-        },
         parser: astParser,
     };
 });
@@ -308,7 +287,40 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
 // Following code is for auto-completion. Reference:
 //   https://codemirror.net/demo/complete.html
 
-const initHints = function() {
+CodeMirror.defineOption('uboHints', null, (cm, hints) => {
+    if ( hints instanceof Object === false ) { return; }
+    if ( Array.isArray(hints.redirectResources) ) {
+        for ( const [ name, desc ] of hints.redirectResources ) {
+            const displayText = desc.aliasOf !== ''
+                ? `${name} (${desc.aliasOf})`
+                : '';
+            if ( desc.canRedirect ) {
+                redirectNames.set(name, displayText);
+            }
+            if ( desc.canInject && name.endsWith('.js') ) {
+                scriptletNames.set(name.slice(0, -3), displayText);
+            }
+        }
+    }
+    if ( Array.isArray(hints.preparseDirectiveEnv)) {
+        preparseDirectiveEnv.length = 0;
+        preparseDirectiveEnv.push(...hints.preparseDirectiveEnv);
+    }
+    if ( Array.isArray(hints.preparseDirectiveHints)) {
+        preparseDirectiveHints.push(...hints.preparseDirectiveHints);
+    }
+    if ( Array.isArray(hints.originHints) ) {
+        originHints.length = 0;
+        for ( const hint of hints.originHints ) {
+            originHints.push(hint);
+        }
+    }
+    if ( hintHelperRegistered ) { return; }
+    hintHelperRegistered = true;
+    initHints();
+});
+
+function initHints() {
     const astParser = new sfp.AstFilterParser({
         interactive: true,
         nativeCssHas: vAPI.webextFlavor.env.includes('native_css_has'),
@@ -613,7 +625,7 @@ const initHints = function() {
         }
         return getOriginHints(cursor, line);
     });
-};
+}
 
 /******************************************************************************/
 
@@ -706,6 +718,9 @@ CodeMirror.registerHelper('fold', 'ubo-static-filtering', (( ) => {
                 case sfp.AST_ERROR_DOMAIN_NAME:
                     msg = `${msg}: Bad domain name`;
                     break;
+                case sfp.AST_ERROR_OPTION_BADVALUE:
+                    msg = `${msg}: Bad value assigned to a valid option`;
+                    break;
                 case sfp.AST_ERROR_OPTION_DUPLICATE:
                     msg = `${msg}: Duplicate filter option`;
                     break;
@@ -715,6 +730,9 @@ CodeMirror.registerHelper('fold', 'ubo-static-filtering', (( ) => {
                 case sfp.AST_ERROR_IF_TOKEN_UNKNOWN:
                     msg = `${msg}: Unknown preparsing token`;
                     break;
+                case sfp.AST_ERROR_UNTRUSTED_SOURCE:
+                    msg = `${msg}: Filter requires trusted source`;
+                    break;
                 default:
                     if ( astParser.isCosmeticFilter() && astParser.result.error ) {
                         msg = `${msg}: ${astParser.result.error}`;
@@ -990,8 +1008,12 @@ CodeMirror.registerHelper('fold', 'ubo-static-filtering', (( ) => {
         }
     };
 
-    self.addEventListener('filterOnHeaders', ( ) => {
-        astParser.options.filterOnHeaders = filterOnHeaders;
+    self.addEventListener('trustedSource', ev => {
+        astParser.options.trustedSource = ev.detail;
+    });
+
+    self.addEventListener('trustedScriptletTokens', ev => {
+        astParser.options.trustedScriptletTokens = ev.detail;
     });
 
     CodeMirror.defineInitHook(cm => {
