@@ -89,13 +89,10 @@ const cmEditor = new CodeMirror(document.querySelector('.codeMirrorContainer'),
 
 vAPI.messaging.send('dashboard', {
     what: 'getAutoCompleteDetails'
-}).then(response => {
+}).then(hints => {
     // For unknown reasons, `instanceof Object` does not work here in Firefox.
-    if ( typeof response !== 'object' ) { return; }
-    const mode = cmEditor.getMode();
-    if ( mode.setHints instanceof Function ) {
-        mode.setHints(response);
-    }
+    if ( hints instanceof Object === false ) { return; }
+    cmEditor.setOption('uboHints', hints);
 });
 
 /******************************************************************************/
