@@ -243,7 +243,7 @@ import {
         if ( typeof hs[key] !== typeof hsDefault[key] ) { continue; }
         this.hiddenSettings[key] = hs[key];
     }
-    this.fireDOMEvent('hiddenSettingsChanged');
+    this.fireEvent('hiddenSettingsChanged');
 };
 
 // Note: Save only the settings which values differ from the default ones.
@@ -259,7 +259,7 @@ import {
     });
 };
 
-self.addEventListener('hiddenSettingsChanged', ( ) => {
+µb.onEvent('hiddenSettingsChanged', ( ) => {
     const µbhs = µb.hiddenSettings;
     ubologSet(µbhs.consoleLogLevel === 'info');
     vAPI.net.setOptions({
@@ -369,11 +369,32 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
 /******************************************************************************/
 
 µb.isTrustedList = function(assetKey) {
-    if ( assetKey.startsWith('ublock-') ) { return true; }
-    if ( assetKey === this.userFiltersPath ) { return true; }
+    if ( this.parsedTrustedListPrefixes.length === 0 ) {
+        this.parsedTrustedListPrefixes =
+            µb.hiddenSettings.trustedListPrefixes.split(/ +/).map(prefix => {
+                if ( prefix === '' ) { return; }
+                if ( prefix.startsWith('http://') ) { return; }
+                if ( prefix.startsWith('file:///') ) { return prefix; }
+                if ( prefix.startsWith('https://') === false ) {
+                    return prefix.includes('://') ? undefined : prefix;
+                }
+                try {
+                    const url = new URL(prefix);
+                    if ( url.hostname.length > 0 ) { return url.href; }
+                } catch(_) {
+                }
+            }).filter(prefix => prefix !== undefined);
+    }
+    for ( const prefix of this.parsedTrustedListPrefixes ) {
+        if ( assetKey.startsWith(prefix) ) { return true; }
+    }
     return false;
 };
 
+µb.onEvent('hiddenSettingsChanged', ( ) => {
+    µb.parsedTrustedListPrefixes = [];
+});
+
 /******************************************************************************/
 
 µb.loadSelectedFilterLists = async function() {
@@ -569,7 +590,7 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
 
     const compiledFilters = this.compileFilters(filters, {
         assetKey: this.userFiltersPath,
-        isTrusted: true,
+        trustedSource: true,
     });
     const snfe = staticNetFilteringEngine;
     const cfe = cosmeticFilteringEngine;
@@ -593,7 +614,7 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
 
     // https://www.reddit.com/r/uBlockOrigin/comments/cj7g7m/
     // https://www.reddit.com/r/uBlockOrigin/comments/cnq0bi/
-    vAPI.net.handlerBehaviorChanged();
+    µb.filteringBehaviorChanged();
 
     vAPI.messaging.broadcast({ what: 'userFiltersUpdated' });
 };
@@ -830,7 +851,7 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
         staticExtFilteringEngine.freeze();
         redirectEngine.freeze();
         vAPI.net.unsuspend();
-        vAPI.net.handlerBehaviorChanged();
+        µb.filteringBehaviorChanged();
 
         vAPI.storage.set({ 'availableFilterLists': µb.availableFilterLists });
 
@@ -908,7 +929,7 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
                 µb.inMemoryFiltersCompiled =
                     µb.compileFilters(µb.inMemoryFilters.join('\n'), {
                         assetKey: 'in-memory',
-                        isTrusted: true,
+                        trustedSource: true,
                     });
             }
             if ( µb.inMemoryFiltersCompiled !== '' ) {
@@ -976,7 +997,7 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
 
     const compiledContent = this.compileFilters(rawDetails.content, {
         assetKey,
-        isTrusted: this.isTrustedList(assetKey),
+        trustedSource: this.isTrustedList(assetKey),
     });
     io.put(compiledPath, compiledContent);
 
@@ -1041,17 +1062,14 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
 
     // Populate the writer with information potentially useful to the
     // client compilers.
+    const trustedSource = details.trustedSource === true;
     if ( details.assetKey ) {
         writer.properties.set('name', details.assetKey);
-        writer.properties.set('isTrusted', details.isTrusted === true);
+        writer.properties.set('trustedSource', trustedSource);
     }
     const assetName = details.assetKey ? details.assetKey : '?';
-    const expertMode =
-        details.assetKey !== this.userFiltersPath ||
-        this.hiddenSettings.filterAuthorMode !== false;
     const parser = new sfp.AstFilterParser({
-        expertMode,
-        filterOnHeaders: µb.hiddenSettings.filterOnHeaders,
+        trustedSource,
         maxTokenLength: staticNetFilteringEngine.MAX_TOKEN_LENGTH,
         nativeCssHas: vAPI.webextFlavor.env.includes('native_css_has'),
     });
@@ -1565,7 +1583,7 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
                             'compiled/' + details.assetKey,
                             this.compileFilters(details.content, {
                                 assetKey: details.assetKey,
-                                isTrusted: this.isTrustedList(details.assetKey),
+                                trustedSource: this.isTrustedList(details.assetKey),
                             })
                         );
                     }
